@@ -1,4 +1,5 @@
 import unittest
+from pathlib import Path
 
 import pandas as pd
 import numpy as np
@@ -18,11 +19,12 @@ from explainerdashboard.explainers import ClassifierExplainer
 from explainerdashboard.datasets import titanic_survive, titanic_names
 
 
-class ClassifierBaseExplainerTests(unittest.TestCase):
+class ClassifierBaseExplainerTestsPipeline(unittest.TestCase):
     def setUp(self):
-        X, y = fetch_openml("titanic", version=1, as_frame=True, return_X_y=True)
-        X = X[['age', 'fare', 'embarked', 'sex', 'pclass']]
-        y = y.astype(int)
+        #X, y = fetch_openml("titanic", version=1, as_frame=True, return_X_y=True)
+        df = pd.read_csv(Path.cwd() / "tests" / "cli_assets" / "pipeline_data.csv")
+        X = df[['age', 'fare', 'embarked', 'sex', 'pclass']]
+        y = df['survived'].astype(int)
 
         numeric_features = ['age', 'fare']
         numeric_transformer = Pipeline(steps=[
